#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Sun 27 Jun 2010 03:37:10 PM CST

''' Generate Thumbnail '''

import os
import os.path
import stat
import errno
import fcntl
import magic
import shutil
import Image
import nas.system
from nas.system import copy
import gst
import gobject
from JpegImagePlugin import RAWMODE
from nas.system import get_name_value
from nas.system import unlink
from nas.system import memuse
from info import DEVICEVERSION
from nas.system import _mkdir

info = {'software': ''}
get_name_value(DEVICEVERSION, info)
product_name = os.path.basename(os.path.realpath('/var/www/nas'))
version_number = info['software']

AUDIO_THUMBNAIL = '/var/www/nas/%s/%s/Applications/MyNAS/icons/audio_thumbnail.jpg' % (product_name, version_number)
BAD_THUMBNAIL = '/var/www/nas/%s/%s/Applications/MyNAS/icons/bad_thumbnail.jpg' % (product_name, version_number)
DEFAULT_THUMBNAIL = '/usr/share/nas/loading.png'

VIDEOS = ['application/ogg',
          'application/x-pn-realmedia',
          'application/x-shockwave-flash']
IMAGE_MODE = {
        '1' :   1,
        'L' :   1,
        'P' :   1,
        'RGB':  6,
        'RGBA': 8,
        'CMYK': 8,
        'YCbCr': 6,
        'I' :   10,
        'F' :   10,
        }

THUMB_LIST = ( \
                ".jpeg", ".jpg", ".png", ".gif", ".bmp", ".tif", ".tiff", ".raw", ".tga", ".mpg", ".mpeg", ".mov", ".mp4", ".avi", ".asf", ".wmv", ".rm", ".rmvb", ".3gp", ".mkv", ".flv", ".swf")



def FileOK(path):
    _dir, _file = os.path.split(path)
    if os.path.splitext(_file)[1].lower() not in THUMB_LIST or '/.' in _dir or _file.startswith('.'):
        return False
    return True


def NeedThumb(path):
    if not FileOK(path):
        return False
    thumbpath = ThumbPath(path)[0]
    if os.stat(path).st_size == 0:
        return False
    if os.path.exists(thumbpath) and os.stat(thumbpath).st_size == 0:
        return True
    try:
        if os.path.getmtime(path) < os.path.getmtime(thumbpath):
            return False
    except OSError:
        return True
    return True


def MemOk(filename):
    im = Image.open(filename)
    w, h = im.size
    mem_usage = memuse()
    mem_free = mem_usage['MemFree'] + mem_usage['Cached'] + mem_usage['SwapFree']
    mem_need = w * h * IMAGE_MODE[im.mode] / 1024
    if mem_free < mem_need:
#        print "False : MemFree %dKB, MemNeed %dKB " % (mem_free, mem_need)
        return False
#    print "True : MemFree %dKB, MemNeed %dKB " % (mem_free, mem_need)
    return True


def _symlink(src, dst):
    try:
        unlink(dst)
        os.symlink(src, dst)
    except os.error as error:
        # 'filesystem not support?'
        if error.errno == errno.EPERM:
            shutil.copy(src, dst)
            return True
        return False
    return True

def ThumbPath(path):
    _dir, _file = os.path.split(path)
    thumbdir = os.sep.join((_dir, ".thumbnail"))
    viewdir = os.sep.join((_dir, ".webview"))
    thumbfile = "".join((_file, ".jpg"))
    return (os.sep.join((thumbdir, thumbfile)), os.sep.join((viewdir, thumbfile)))


def CopyThumb(filename):
    thumbfile, viewfile = ThumbPath(filename)
    try:
        mtime = os.path.getmtime(filename)
    except OSError:
        return 
    copy(DEFAULT_THUMBNAIL, thumbfile)
    copy(DEFAULT_THUMBNAIL, viewfile)
    os.utime(thumbfile, (mtime - 100, mtime -100))


def convert(filepath):
    try:
        mode = os.stat(filepath).st_mode
    except os.error as error:
        if error.errno == errno.ENOENT:
            return {}
        return {}
    if stat.S_ISLNK(mode):
        return convert(os.readlink(filepath))
    if not stat.S_ISREG(mode):
        return {}

    mime = magic.open(magic.MAGIC_MIME)
    if mime != None:
        mime.load()

    mimetype = mime.file(filepath).split(';')[0]

    # application/ogg
    # application/x-pn-realmedia
    # application/x-shockwave-flash
    if mimetype[:5] == 'video' or mimetype in VIDEOS:
        filetype = 'video'
        # generate thumbnail
        imagepath = '%s/thumb.data.tmp' % os.path.dirname(filepath)
        result = Thumbnail(filepath, imagepath)
        mimetype = mime.file(imagepath).split(';')[0]
        if mimetype[:5] == 'image': # good
            filetype = 'video'
        elif result.has_audio:
            filetype = 'audio'
        else:
            filetype = 'none'
    elif mimetype[:5] == 'audio':
        filetype = 'audio'
    elif mimetype[:5] == 'image':
        filetype = 'image'
        imagepath = filepath
    else:
        filetype = 'none'

    _dir, name = os.path.split(filepath)
    wdir = os.path.join(_dir, '.webview')
    tdir = os.path.join(_dir, '.thumbnail')
    _mkdir(wdir)
    _mkdir(tdir)
    wname = os.path.join(wdir, name + '.jpg')
    tname = os.path.join(tdir, name + '.jpg')

    try:
        os.unlink(wname)
    except os.error:
        pass

    try:
        os.unlink(tname)
    except os.error:
        pass

    if filetype == 'video' or filetype == 'image':
        result = _convert(imagepath, wname, tname)
        if filetype == 'video':
            unlink(wname)
            del result['.webview']
        if imagepath != filepath:
            unlink(imagepath)
        return result

    if filetype == 'audio':
        thumbnail = AUDIO_THUMBNAIL
    elif filetype == 'none':
        thumbnail = BAD_THUMBNAIL

    _symlink(thumbnail, tname)

    return {'.thumbnail': thumbnail}

def _convert(filepath, wname, tname):

    flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
    wdesc = os.open(wname, flags, 0664)
    tdesc = os.open(tname, flags, 0664)
    fcntl.lockf(wdesc, fcntl.LOCK_EX)
    fcntl.lockf(tdesc, fcntl.LOCK_EX)

    try:
        img = Image.open(filepath)
#        img.load()
    except IOError as error:
        if error.errno is not None:
            fcntl.lockf(wdesc, fcntl.LOCK_UN)
            fcntl.lockf(tdesc, fcntl.LOCK_UN)
            os.close(wdesc)
            os.close(tdesc)
            return {'.webview': BAD_THUMBNAIL,
                    '.thumbnail': BAD_THUMBNAIL}

    (width, height) = img.size
    scale = float(height) / width
    if scale < 0.75:
        wsize = [(int(800 * x), 1)[int(800 * x) == 0] for x in (1, scale)]
        tsize = [(int(128 * x), 1)[int(128 * x) == 0] for x in (1, scale)]
    elif scale < 1:
        wsize = [(int(600 / x), 1)[int(600 / x) == 0] for x in (scale, 1)]
        tsize = [(int(128 * x), 1)[int(128 * x) == 0] for x in (1, scale)]
    else:
        wsize = [(int(600 / x), 1)[int(600 / x) == 0] for x in (scale, 1)]
        tsize = [(int(128 / x), 1)[int(128 / x) == 0] for x in (scale, 1)]


    try:
        img.thumbnail(wsize, Image.BILINEAR)

        img = img.convert('RGB')
#        if 'transparency' in img.info or img.mode == 'RGBA':
#            new = img.mode == 'RGBA' and img or img.convert('RGBA')
#            img = Image.new('RGB', new.size)
#            img.paste(new, (0, 0), new)
#        elif img.mode not in RAWMODE:
#            img = img.convert('RGB')

        wfile = os.fdopen(wdesc, 'w')
        img.save(wfile, 'JPEG', quality=50)
        wfile.flush()
    except Exception:
        fcntl.lockf(wdesc, fcntl.LOCK_UN)
        fcntl.lockf(tdesc, fcntl.LOCK_UN)
        os.close(wdesc)
        os.close(tdesc)
        return {'.webview': BAD_THUMBNAIL,
                '.thumbnail': BAD_THUMBNAIL}

    # we have save webview, thumbnail shouldn't have any problem
    img.thumbnail(tsize, Image.BILINEAR)
    tfile = os.fdopen(tdesc, 'w')
    img.save(tfile, 'JPEG', quality=50)
    tfile.flush()

    os.fsync(wdesc)
    os.fsync(tdesc)
    fcntl.lockf(wdesc, fcntl.LOCK_UN)
    fcntl.lockf(tdesc, fcntl.LOCK_UN)
    wfile.close()
    tfile.close()

    return {'.webview': wname,
            '.thumbnail': tname}

class Thumbnail:

    def __init__(self, location, thumbnail, second=None):
        gobject.threads_init()

        # get abs path
        location = os.path.abspath(location)
        self.thumbnail = os.path.abspath(thumbnail)
        if not os.path.isfile(location):
            return None

        self.seeked = False
        self.playing = False
        self.has_video = False
        self.has_audio = False
        self.width = None
        self.height = None
        self.length = -1

        self.second = isinstance(second, (int, float)) and second or 0

        self.pipeline = gst.Pipeline()

        bus = self.pipeline.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self.on_message)

        src = gst.element_factory_make('filesrc')
        src.set_property('location', location)
        decode = gst.element_factory_make('decodebin')
        decode.connect('new-decoded-pad', self.on_new_decoded_pad)
        decode.connect('no-more-pads', self.on_no_more_pads)

        # video
        ffmpeg = gst.element_factory_make('ffmpegcolorspace')
        # ffmpeg = gst.element_factory_make('videoscale')
        pngenc = gst.element_factory_make('pngenc')
        filesink = gst.element_factory_make('filesink')
        filesink.set_property('location', self.thumbnail)
        self.pipeline.add(src, decode, ffmpeg, pngenc, filesink)

        src.link(decode)
        gst.element_link_many(ffmpeg, pngenc, filesink)

        self.video = ffmpeg.get_pad('sink')
        self.video.connect("notify::caps", self.on_notify_caps)

        self.pipeline.set_state(gst.STATE_PAUSED)
        self.mainloop = gobject.MainLoop()
        self.mainloop.run()

    def on_new_decoded_pad(self, decode, pad, last):
        caps = pad.get_caps()
        # name = caps[0].get_name()
        if 'video' in caps.to_string():
            self.has_video = True
            if not self.video.is_linked():
                pad.link(self.video)
            self.pipeline.set_state(gst.STATE_PLAYING)
            self.playing = True
        if 'audio' in caps.to_string():
            self.has_audio = True

    def on_no_more_pads(self, decode):
        if not self.playing:
            self.pipeline.set_state(gst.STATE_PLAYING)

    def on_message(self, bus, message):
        if message.type in [gst.MESSAGE_EOS, gst.MESSAGE_ERROR]:
            self.pipeline.set_state(gst.STATE_NULL)
            self.mainloop.quit()
            #if not self.has_video:
            #    os.unlink(self.thumbnail)
        #elif message.type == gst.MESSAGE_TAG:
        #    tags = message.parse_tag()
        #    if 'duration' in tags:
        #        self.length = tags['duration']
        if not self.seeked and self.second:
            position = self.second * gst.SECOND
            flag = gst.SEEK_FLAG_FLUSH
            if self.length > position:
                self.seeked = True
                self.pipeline.seek_simple(gst.FORMAT_TIME, flag, position)
            elif self.length != -1:
                self.seeked = True
                self.pipeline.seek_simple(gst.FORMAT_TIME, flag, self.length)

    def on_notify_caps(self, pad, arg):
        caps = pad.get_negotiated_caps()
        if not caps:
            return None
        if 'video' in caps.to_string():
            self.width = caps[0]['width']
            self.height = caps[0]['height']
        query = gst.query_new_duration(gst.FORMAT_TIME)
        if pad.get_peer().query(query):
            gst.FORMAT_TIME, self.length = query.parse_duration()

__all__ = ['convert']


# vim: set sta sw=4 et:
