#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 04 Aug 2010 10:47:49 AM CST

import os
import stat
import time
import zipfile

from system import NASError
from system import getrealpath

from users import _checkhomedir

def _getdirname(path):
    try:
        sb = os.stat(path)
    except os.error:
        return None
    if stat.S_ISDIR(sb.st_mode):
        return path.split(os.sep)
    else:
        return path.split(os.sep)[:-1]

def _getparent(path, parent):
    if not parent:
        return os.sep.join(_getdirname(path))
    dirs = _getdirname(path)
    if dirs is None:
        return parent
    parents = _getdirname(parent)
    maxindex = min(len(dirs), len(parents))
    index = 0
    while index < maxindex:
        if dirs[index] != parents[index]:
            break
        index += 1
    return os.sep.join(parents[:index])

def zipzip(path=None, login=None, name=None, encoding='utf8', **kw):
    if login is None or path is None:
        raise NASError('Bad Request')

    paths = []
    if not isinstance(path, list):
        path = [path]

    parent = ''
    for item in path:
        realpath = getrealpath(item, login, None)
        if not realpath:
            continue
        parent = _getparent(realpath, parent)
        if os.path.isdir(realpath):
            try:
                for root, dirs, files in os.walk(realpath):
                    if os.path.split(root)[1][:1] == '.':
                        continue
                    for _file in files:
                        if _file[:1] == '.':
                            continue
                        realpath = os.path.join(root, _file)
                        if realpath not in paths:
                            paths.append(realpath)
            except:
                pass
        elif realpath not in paths:
            paths.append(realpath)

    if len(paths) == 0:
        raise NASError('No File to zipped...')

    if name is None:
        name = time.strftime('%Y%m%d%H%M%S.zip')

    homedir = _checkhomedir(login)
    zipname = '%s/%s' % (homedir, name)
    zipurl = '%s/%s?login=%s' % ('/dav/home', name, login)

    ZIP = zipfile.ZipFile(zipname, 'w')
    for path in paths:
        if path == zipname:
            continue
        if isinstance(path, unicode):
            arcname = path.replace(parent, '', 1)
        elif isinstance(path, str):
            arcname = path.replace(parent, '', 1).decode('utf8', 'ignore')
        else:
            continue
        arcname = arcname.encode(encoding, 'xmlcharrefreplace')

        ZIP.write(path, arcname)

    ZIP.close()
    return {'zipfile': zipname, 'location': zipurl}
    # return zipname, zipurl

# vim: set sta sw=4 et:
