#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Tue 16 Nov 2010 04:00:32 PM CST

''' flickr upload photo '''

import httplib
import mimetypes
mimetypes.init()

KEY = 'bb9ad1b48280c3348d1fbbf79fb82afd'
SECRET = '483833773cc7e100'

HOST = 'secure.flickr.com'
PORT = 443
REST = '/services/rest/'
AUTH = '/services/auth/'
UPLOAD = '/services/upload/'

from system import NASError
from system import getrealpath
from facebook import _multipart

import socket
from hashlib import md5
from xml.dom import Node
from xml.dom.minidom import parseString

class FlickrError(Exception):
    def __init__(self, detail):
        raise NASError('Flickr Error', detail)

def signature(query):
    query['api_key'] = KEY
    keys = query.keys()
    keys.sort()
    sig = SECRET
    for key in keys:
        if key == 'api_sig' or not isinstance(query.get(key), str):
            continue
        sig += '%s%s' % (key, query.get(key))
    query['api_sig'] = md5(sig).hexdigest()

def getnodevalue(xml, tagname, default=''):
    def getpcdata(xml, default=''):
        ''' get xml pcdata '''
        for node in xml.childNodes:
            if node.nodeType  == Node.TEXT_NODE:
                return node.data.encode('utf8')
            if node.nodeType == Node.CDATA_SECTION_NODE:
                return node.data.encode('utf8')
        return default

    node = xml.getElementsByTagName(tagname)
    if len(node) == 1 and node[0].nodeType == Node.ELEMENT_NODE:
        return getpcdata(node[0], default)
    return default

def getattribute(xml, attrname, default=''):
    attr = xml.getAttributeNode(attrname)
    if attr:
        return attr.value.encode('utf8')
    return default

def _postdata(path, body, headers=None):
    connection = httplib.HTTPSConnection(HOST, PORT)
    connection.putrequest('POST', path)

    has_type = False
    has_length = False
    if headers is not None:
        for (name, value) in headers:
            if name.lower() == 'content-length':
                has_length = True
            if name.lower() == 'content-type':
                has_type = True
            connection.putheader(name, value)

    if not has_type:
        connection.putheader('Content-Type',
                             'application/x-www-form-urlencoded')

    if not has_length:
        connection.putheader('Content-Length', str(len(body)))

    try:
        connection.endheaders()
    except socket.error as error:
        raise NASError('Proxy Error', error.strerror)

    connection.send(body)
    if hasattr(body, 'close'):
        body.close()

    return _getresult(connection)

def _getresult(connection):

    try:
        response = connection.getresponse()
    except httplib.HTTPException as error:
        connection.close()
        raise NASError('Proxy Error', error.reason)

    content = response.read()
    connection.close()
    if response.status != 200:
        raise FlickrError('Invalid Status')

    try:
        xml = parseString(content)
    except Exception:
        raise FlickrError('Invalid Content')

    rsp = xml.getElementsByTagName('rsp')
    if len(rsp) != 1:
        raise FlickrError('Invalid Content')

    stat = getattribute(rsp[0], 'stat')
    if stat == 'fail':
        err = rsp[0].getElementsByTagName('err')
        if len(err) != 1:
            raise FlickrError('Invalid Content')
        # code = getattribute(err[0], 'code', '')
        msg = getattribute(err[0], 'msg', '')
        raise FlickrError(msg)

    return rsp[0]

def uploadphoto(path=None, login=None, **kw):
    arguments = ['auth_token', 'title', 'description', 'tags',
                 'is_public', 'is_friend', 'is_family',
                 'safety_level', 'content_type', 'hidden']

    realpath = getrealpath(path, login, False)
    if not realpath:
        raise NASError('Not Found')

    content_type = mimetypes.guess_type(realpath)[0]
    if not content_type.startswith('image'):
        raise NASError('Not Image')

    query = {'photo': open(realpath)}
    for argument in arguments:
        if kw.get(argument):
            query[argument] = kw.get(argument)

    signature(query)

    headers, body = _multipart(query)
    data = _postdata(UPLOAD, body, headers)
    return {'photoid': getnodevalue(data, 'photoid')}

__all__ = ['uploadphoto']

# vim: set sta sw=4 et:
