#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 08:10:11 PM CST

''' Share Folder '''

from system import NASError
from system import _all, _any
from system import getrealpath
from system import check_table
from system import sqlite, HOMEDB
from system import getlist, getunicode

import time
import urllib
from cStringIO import StringIO

from nas.users import _getusers

TZ = '%Y-%m-%dT%H:%M:%SZ'
RFC1123 = '%a, %d %b %Y %H:%M:%S GMT'
DIR_MIMETYPE = 'httpd/unix-directory'
DTNS = 'xmlns:dt="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/"'

def getfolderowner(login=None, **kw):

    login = getunicode(login)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id, name, path FROM folder WHERE
                    owner = ? ORDER BY name ASC
                ''', (login,))

    body = StringIO()
    body.write('<?xml version="1.0" encoding="utf-8"?>')
    body.write('<D:multistatus xmlns:D="DAV:" %s>' % DTNS)

    users = _getusers()

    for folder in cur.fetchall():
        name = folder[1]
        path = folder[2]
        if not getrealpath(path, login, True):
            cur.execute('DELETE FROM folder WHERE id = ?', (folder[0],))
            cur.execute('DELETE FROM perm WHERE folder = ?', (folder[0],))
            continue

        body.write('<D:response>')
        body.write('<D:href>')
        if isinstance(path, unicode):
            path = path.encode('utf8')
        body.write(urllib.quote(path))
        body.write('</D:href>')
        body.write('<D:propstat>')
        body.write('<D:prop>')

        body.write('<D:resourcetype><D:collection/></D:resourcetype>')
        body.write('<D:getcontenttype>%s</D:getcontenttype>' % DIR_MIMETYPE)

        body.write('<D:creationdate dt:dt="dateTime.tz">')
        body.write(time.strftime(TZ, time.gmtime()))
        body.write('</D:creationdate>')

        body.write('<D:getlastmodified dt:dt="dateTime.rfc1123">')
        body.write(time.strftime(RFC1123, time.gmtime()))
        body.write('</D:getlastmodified>')

        body.write('<D:getcontentlength>0</D:getcontentlength>')
        body.write('<D:getcontentlanguage>en</D:getcontentlanguage>')

        body.write('<D:displayname>%s</D:displayname>' % name.encode('utf8'))

        cur.execute(''' SELECT sharer, write FROM perm WHERE
                        folder = ? ORDER BY sharer ASC
                    ''', (folder[:1]))
        for share in cur.fetchall():
            if share[0].encode('utf8') not in users:
                continue
            write = share[1] and 'readwrite' or 'readonly'
            body.write('<R:sharer xmlns:R="works" write="%s">' % write)
            body.write(share[0].encode('utf8'))
            body.write('</R:sharer>')

        body.write('</D:prop>')
        body.write('<D:status>HTTP/1.1 200 OK</D:status>')
        body.write('</D:propstat>')
        body.write('</D:response>')

    body.write('</D:multistatus>')
    con.commit()
    con.close()

    return body

def _getperms(share, write):
    shares = getlist(share)
    writes = []
    for item in getlist(write):
        if item.lower() == u'readwrite':
            writes.append(1)
        elif item.lower() == u'readonly':
            writes.append(0)
        else:
            writes.append(None)

    perms = zip(shares, writes)
    if u'*' in shares:
        for perm in perms:
            if perm[0] == u'*':
                perms = [perm]
                break
    return perms

def addfolder(login=None, name=None, path=None,
        sharer=None, write=None, **kw):
    if not _all(login, name, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    path = getunicode(path)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    try:
        cur.execute(''' INSERT INTO folder (owner, name, path)
                        VALUES (?, ?, ?)
                    ''', (login, name, path))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Folder Exists', name.encode('utf8'))
    cur.execute(''' SELECT id FROM folder
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    folder = cur.fetchone()[0]
    con.commit()
    con.close()

    if _any(sharer, write):
        perms = _getperms(sharer, write)
        _modifyperm(folder, perms)

    return {'folder': 'add folder'}

def editfolder(login=None, path=None, name=None,
        sharer=None, write=None, **kw):
    if not _all(login, path):
        raise NASError('Bad Request')

    if not _any(name, sharer, write):
        raise NASError('Bad Request')

    path = getunicode(path)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM folder
                    WHERE owner = ? AND path = ?
                ''', (login, path))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Folder Not Exists', path.encode('utf8'))
    folder = result[0]

    if name is not None:
        name = getunicode(name)
        try:
            cur.execute(''' UPDATE folder SET name = ?  WHERE id = ?
                        ''', (name, folder))
        except sqlite.IntegrityError:
            con.close()
            raise NASError('Folder Exists', name.encode('utf8'))
        con.commit()
        con.close()

    if _any(sharer, write):
        perms = _getperms(sharer, write)
        _modifyperm(folder, perms)

    return {'folder': 'update folder'}

def delfolder(login=None, path=None, **kw):
    if not _all(login, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    path = getunicode(path)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM folder
                    WHERE owner = ? AND path = ?
                ''', (login, path))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Folder Not Exists', path.encode('utf8'))
    folder = result[0]
    cur.execute(''' DELETE FROM folder WHERE id = ?
                ''', (folder,))
    con.commit()
    con.close()

    _delperm(folder)

    return {'folder': 'delete folder'}

def _modifyperm(folder, newperms):
    if newperms[0] == (u'*', None):
        _delperm(folder)
        return True

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT sharer, write FROM perm WHERE folder = ?
                ''', (folder,))
    oldperms = cur.fetchall()
    con.close()

    for perm in oldperms:
        if perm in newperms:
            newperms.remove(perm)
            oldperms.remove(perm)

    for perm in oldperms:
        _delperm(folder, perm[0])

    for perm in newperms:
        if perm[1] is not None:
            _addperm(folder, perm[0], perm[1])
        else:
            _delperm(folder, perm[0])

    return True

def _delperm(folder=None, sharer=None):
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    if sharer is None:
        cur.execute(''' DELETE FROM perm WHERE folder = ?
                    ''', (folder,))
    else:
        cur.execute(''' DELETE FROM perm WHERE folder = ? AND sharer = ?
                    ''', (folder, sharer))

    con.commit()
    con.close()
    return True

def _addperm(folder=None, sharer=None, write=None):
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    try:
        cur.execute(''' INSERT INTO perm (folder, sharer, write)
                        VALUES (?, ?, ?)
                    ''', (folder, sharer, write))
    except sqlite.IntegrityError:
        cur.execute(''' UPDATE perm SET write = ?
                        WHERE folder = ? AND sharer = ?
                        ''', (write, folder, sharer))
    con.commit()
    con.close()

    return True

__all__ = ['getfolderowner', 'addfolder', 'editfolder', 'delfolder']

# vim: set sta sw=4 et:
