#!/usr/bin/env python
#fileencoding: utf8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 14 Apr 2010 09:35:01 AM CST

WEBDIR = '/var/www/nas'
WEBUSER = 'lighttpd'

ADMIN = 'admin'

TZ = '%Y-%m-%dT%H:%M:%SZ'
RFC1123 = '%a, %d %b %Y %H:%M:%S GMT'
HOMEDB = '/home/.nas/sqlite.db'
SESSIONDB = '/etc/nas/sqlite.db'
WORKSR = 'http://isharing.workssys.com/2010'
WEBROOT = '/var/www/nas'

import os
import stat
import sys
import pwd
import grp
import fcntl
import errno
import shutil
import urllib
import shlex
from subprocess import Popen, PIPE
from sqlite3 import dbapi2 as sqlite

import re
import json
from cStringIO import StringIO

def parse(data):
    params = [line.partition(':') for line in data.split('\r\n') if line]
    params = [(k.strip(), v.strip()) for k, _, v in params if k]
    return dict(params)

def escape(cmd):
    return '\r\n'.join(['%s:%s' % (k.strip(), v.strip()) for k, v in cmd]) + "\r\n\r\n"


def mkdirtree(path):
    _dir = os.path.split(path)[0].split('/')[1:]
    dir_path = ''
    for elem in _dir:
        dir_path = os.sep.join([dir_path, elem])
        if os.path.exists(dir_path):
            continue
        try:
            os.mkdir(dir_path)
        except:
            return False
    return True

def GetSockproxyStatus(**kw):
    status = {}
    try:
        status['status'] =  file('/var/lib/sockproxy/server').read().split(':')[-1]
    except:
        return 'False'
    return status

def copy(src, dst):
    unlink(dst)
    _mkdir(os.path.split(dst)[0])
    shutil.copy(src, dst)
    return True

def unlink(name):
    try:
        mode = os.stat(name).st_mode
    except os.error as error:
        if error.errno == errno.ENOENT:
            return True

    if stat.S_ISDIR(mode):
        delete = os.rmdir
    else:
        delete = os.unlink

    try:
        delete(name)
    except os.error:
        return False

    return True


def POPEN(command, inputs=None, outputs=PIPE, errors=PIPE):
    '''
    run a system command, return essential information

    (code, readble-object)
    code = 0, succes, stdout
    code != 0, fail, stderr
    '''
    #os.setuid(0)
    #os.setgid(0)

    os.environ['PATH'] = '/bin:/sbin:/usr/bin:/usr/sbin'
    if isinstance(command, list):
        args = command
    else:
        args = shlex.split('%s' % command)

    print >> sys.stderr, '[command debug] %s' % args

    try:
        popen = Popen(args, stdin=inputs, stdout=outputs, stderr=errors)
        stdout = popen.stdout
        stderr = popen.stderr
        popen.wait()
        code = popen.returncode
    except OSError as error: # errno, strerror, filename
        data = StringIO()
        data.write(error.strerror)
        data.seek(0)
        return (error.errno, data)

    def _close(_file):
        if hasattr(_file, 'close'):
            _file.close()

    if code == 0:
        if args[0] == 'exportfs':
            _close(stdout)
            return (code, stderr)
        else:
            _close(stderr)
            return (code, stdout)
    else:
        if args[0] == 'su':
            _close(stderr)
            return (code, stdout)
        else:
            _close(stdout)
            return (code, stderr)

def get_name_value(_file, names, comms=None, equal='='):
    '''
    parse name values from file

    a=b
    a=c
    ==> a = c or a = ['b', 'c']

    names -> dictionary
    names[item] -> value
         value, list, => list
                else, => str

    comment => comms, #%
    equal, '='

    resolve.conf
    nameserver ....
    '''
    if comms is None:
        comms = ['#']

    if not os.path.exists(_file):
        return False

    pipe = open(_file)
    while True:
        line = pipe.readline()
        if not line:
            break
        for comm in comms:
            line = line.partition(comm)[0].strip()
        if not line:
            continue
        (head, sep, tail) = line.partition(equal)
        if sep != equal:
            continue
        name = head.strip()
        value = tail.strip()
        if name in names:
            if isinstance(names[name], list):
                names[name].append(value)
            else:
                names[name] = value
    pipe.close()
    return True

def set_name_value(_file, names, comms=None, equal='='):
    '''
    set name value into files
    please c.f. get_name_value

    names[item] = None, delete item

    if file is modified, return True
    '''
    if comms is None:
        comms = ['#']

    if os.path.exists(_file):
        pipe = open(_file, 'r')
        read = True
    else:
        pipe = open(_file, 'w')
        read = False

    change = False

    data = StringIO()
    while read:
        orig = pipe.readline()
        if not orig:
            break
        line = orig
        for comm in comms:
            line = line.partition(comm)[0].strip()
        if not line:
            data.write(orig)
            continue
        (head, sep, tail) = line.partition(equal)
        name = head.strip()
        value = tail.strip()
        if sep != equal:
            if name not in names:
                data.write(orig)
            continue
        if name in names:
            if isinstance(names[name], list):
                orig = ''
            elif value != names[name]:
                change = True
                if names[name] is None:
                    orig = ''
                else:
                    orig = '%s%s%s\n' % (name, equal, names[name])
                names[name] = ''
            else:
                names[name] = ''
        data.write(orig)
    pipe.close()

    for name in names:
        if isinstance(names[name], list):
            for value in names[name]:
                if value is not None:
                    change = True
                    if value:
                        data.write('%s%s%s\n' % (name, equal, value))
        elif names[name] is not None:
            change = True
            if names[name]:
                data.write('%s%s%s\n' % (name, equal, names[name]))

    content = data.getvalue()
    data.close()

    if change:
        backup(_file)
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
        pipe = os.open('%s.2' % _file, flags, 0644)
        fcntl.lockf(pipe, fcntl.LOCK_EX)
        ret = os.write(pipe, content)
        os.fsync(pipe)
        fcntl.lockf(pipe, fcntl.LOCK_UN)
        os.close(pipe)
        if ret != len(content):
            raise NASError('write data error', _file)
        os.rename('%s.2' % _file, _file)
        unbackup(_file)
        return True

    return None

def pcdata(text):
    '''
    xml pcdata, replace &,<,>,"
    '''
    if isinstance(text, unicode):
        text = text.encode('utf8')
    text = text.replace('&', '&amp;').replace('<', '&lt;')
    text = text.replace('>', '&gt;').replace('"', '&quot;')
    return text
    # invalid = '[\x00-\x08\x0b\x0c\x0e-\x1f]'
    # return re.sub(invalid, '', text)

def writexml(value, script='false', output='xml', callback='callback', fastcgi=True, **kw):
    '''
    value, string, list, dict

    convert value to xml or json

    output: xml | json

    json:
       script,
       callback
    '''

    data = StringIO()

    def validtag(tagname):
        '''
        check tagname is or not a valid tag
        '''
        if tagname is None:
            return None
        if isinstance(tagname, unicode):
            tagname = tagname.encode('utf8')
        elif not isinstance(tagname, str):
            return False
        if re.match(r'^[:A-Z_a-z][:A-Z_a-z-.0-9]*$', tagname):
            return tagname
        return False

    def genxml(stringio, tag, value):
        '''
        generate xml with tag, value
        '''
        tag = validtag(tag)
        if tag is False:
            return None
        if isinstance(value, basestring):
            if tag is None:
                stringio.write('%s' % pcdata(value))
            elif value:
                stringio.write('<%s>%s</%s>' % (tag, pcdata(value), tag))
            else:
                stringio.write('<%s/>' % tag)
        elif isinstance(value, list):
            for v in value:
                if not v:
                    value.remove(v)
            if not value:
                return None
            while value:
                genxml(stringio, tag, value.pop(0))
        elif isinstance(value, dict):
            if tag is not None:
                stringio.write('<%s>' % tag)
            while value:
                k, v = value.popitem()
                genxml(stringio, k, v)
            if tag is not None:
                stringio.write('</%s>' % tag)

    if hasattr(value, 'getvalue'):
        data.write(value.getvalue())
        value.close()
    elif isinstance(value, (basestring, list, dict)) and output == 'json':
        if script == 'true':
            data.write('<script>')
        data.write('%s(' % str(callback))
        json.dump(value, data)
        data.write(')')
        if script == 'true':
            data.write('</script>')
    elif isinstance(value, (basestring, list, dict)):
        data.write('<?xml version="1.0" encoding="utf-8"?>')
        data.write('<nas>')
        genxml(data, None, value)
        data.write('</nas>')
    content = data.getvalue()

    data.close()
    if fastcgi:
        return [content]
    else:
        sys.stdout.write(content)

def getgid(name):
    _file = open('/etc/group')
    while True:
        line = _file.readline()
        if not line:
            _file.close()
            return 0
        if line.startswith(name + ':'):
            gid = int(line.split(':')[2])
            _file.close()
            return gid

def daemon(stdin=os.devnull, stdout=os.devnull, stderr=os.devnull):
    # Adapted from Chad J. Schroeder's recipe
    # @see http://code.activestate.com/recipes/278731/
    pid = os.fork()
    if pid == 0:
        # parent 2
        os.setsid()
        pid = os.fork()
        if pid == 0:
            # child
            os.chdir('/')
            os.umask(0)
        else:
            # parent 2
            os._exit(0)
    else:
        # parent 1
        os._exit(0)

    fd_in = os.open(stdin, os.O_RDONLY)
    os.dup2(fd_in, 0)
    fd_out = os.open(stdout, os.O_WRONLY | os.O_CREAT)
    os.dup2(fd_out, 1)
    fd_err = os.open(stderr, os.O_WRONLY | os.O_CREAT)
    os.dup2(fd_err, 2)

def getpid(pidfile):
    if os.path.exists(pidfile):
        _file = open(pidfile)
        try:
            pid = int(_file.readline().strip())
        except ValueError:
            pid = None
        finally:
            _file.close()
        if pid:
            try:
                os.kill(pid, 0)
            except OSError as err:
                if err.errno == errno.ESRCH:
                    pid = None
                    os.unlink(pidfile)
                # ... else ? why ?
        return pid
    return None

def getunicode(param):
    if isinstance(param, unicode):
        return param
    else:
        return str(param).decode('utf8', 'ignore')

def getlist(params, unquote=False):
    if isinstance(params, list):
        result = []
        for param in params:
            param, nil, nil = param.partition('?')
            if isinstance(param, str):
                if unquote:
                    param = urllib.unquote(param)
                result.append(param.decode('utf8', 'ignore'))
            elif isinstance(param, unicode):
                result.append(param)
        return result
    elif isinstance(params, str) and params:
        if unquote:
            params = urllib.unquote(params)
        params, nil, nil = params.partition('?')
        return [params.decode('utf8', 'ignore')]
    elif isinstance(params, unicode) and params:
        params, nil, nil = params.partition(u'?')
        return [params]
    else:
        return [None]

def fixperm(path):
    try:
        mode = os.stat(path).st_mode
    except os.error: return
    if stat.S_ISDIR(mode):
        if not (mode & stat.S_IXUSR and mode & stat.S_IXGRP and mode & stat.S_IXOTH):
            try:
                os.chmod(path, mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
            except:
                pass
    elif stat.S_ISREG(mode):
        if not (mode & stat.S_IRUSR and mode & stat.S_IRGRP and mode & stat.S_IROTH):
            try:
                os.chmod(path, mode | stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH)
            except:
                pass
    else: return
    parent = os.path.split(os.path.realpath(path))[0]
    if parent != '/':
        fixperm(parent)

class NASError(Exception):
    ''' NAS Error '''
    def __init__(self, *kw):
        if len(kw) == 0:
            self.reason = 'Unknown Error'
            self.detail = ''
        elif len(kw) == 1:
            self.reason = kw[0]
            self.detail = ''
        elif len(kw):
            self.reason = kw[0]
            self.detail = kw[1]
        self.args = kw

# public -> /home/public
# share -> ...
# home -> /home/[login]
# device -> /media
# device/flash -> /home

ROOT = ''
HOME = '/home'
SHARE = '/share'

PUBLIC = '/public'
REALPUBLIC = '/home/Public'
HIDEHOMES = 'Public'

DEVICE = '/device'
REALDEVICE = '/media'

HOMES = '/device/homes'
REALHOMES = '/home'

FIRMWARE = '/firmware'
BACKUP = '/backup'
RESTORE = '/restore'
REALRESTORE = '/home/.nas/restore.zip'
REALFIRMWARE = '/var/cache/nas/manual/firmware.bin'

DOWNLOADS = '/downloads'
REALDOWNLOADS = '/home/downloads'

PUBLISH = '/publish'

def _canwrite(path=REALHOMES):
    '''
    check whether a path can be write.
    '''
    if not path:
        return False

    def samedev(path, dev, attr):
        try:
            sb = os.stat(path)
        except:
            return None
        if getattr(sb, attr) == dev:
            return True
        return False

    parent = os.path.split(path)[0]
    try:
        sb = os.stat(path)
    except os.error:
        return _canwrite(parent)

    if stat.S_ISBLK(sb.st_mode):
        dev = sb.st_rdev
    else:
        dev = sb.st_dev

    mntent = open('/proc/mounts')
    while True:
        line = mntent.readline()
        try:
            _fsname, _dir, _fstype = line.split()[:3]
        except ValueError:
            _fsname, _dir, _fstype = None, None, None
            if not line:
                break
            continue

        if _fsname == 'rootfs':
            continue

        if path in (_fsname, _dir):
            break

        if samedev(_dir, dev, 'st_dev'):
            break

        if samedev(_fsname, dev, 'st_rdev'):
            break

    mntent.close()
    if _fstype in (None, 'ubifs'):
        return False
    return True

try:
    ftp = pwd.getpwnam('ftp')
    ftpuid = ftp.pw_uid
except KeyError:
    ftpuid = 0
def url2path(path, login, query='', check=False, kw=None):
    '''
    dav/home -> /home/user
    dav/devices -> /media
    dav/share -> .... sqlitedb
    dav/publish -> .... sqlitedb
    dav/public -> /home/Public
    dav/album -> ...

    (realpath, isdir, writable)
    '''

    if isinstance(path, unicode):
        path = path.encode('utf8')

    path = urllib.unquote(path)
    # tail = path[-1:]
    path = os.path.normpath(path)

    from album import album2path
    path, login = album2path(path, login)
    if path is None:
        raise PathError(404)

    if path[:4] == '/dav':
        path = path[4:]
    elif path[:7] == '/webdav':
        path = path[7:]
    else:
        raise PathError(403)

    write = None
    if path == ROOT:
        return (ROOT, True, write)
    elif path == SHARE:
        return (SHARE, True, write)
    elif path.startswith(FIRMWARE):
        _mkdir(os.path.dirname(REALFIRMWARE))
        return (REALFIRMWARE, False, True)
    elif path.startswith(BACKUP):
        return (BACKUP, False, False)
    elif path.startswith(RESTORE):
        return (REALRESTORE, False, True)
    elif path.startswith(PUBLIC):
        gid = getgid('nas')
        _mkdir(REALPUBLIC)
        chown(REALPUBLIC, 0, gid, recursive=False)
        try:
            os.symlink('/home/admin/Public', REALPUBLIC)
        except OSError:
            pass
        _mkdir(os.path.join(REALPUBLIC, 'upload'))
        chown(os.path.join(REALPUBLIC, 'upload'), ftpuid, gid, recursive=False)
        path = path.replace(PUBLIC, REALPUBLIC, 1)
    elif path.startswith(HOME) and login:
        try:
            pw = pwd.getpwnam(login)
        except KeyError:
            raise NASError('Invalid Address')
        if path == HOME:
            make_standard_userdirs(pw)
        path = path.replace(HOME, pw.pw_dir, 1)
    elif path.startswith(SHARE):
        path = path.replace(SHARE, '', 1)[1:]
        print 'path', path
        path, sep, child = path.partition(os.sep)
        name, sep, owner = path.rpartition('@')
        # name@owner
        if sep == '@':
            con = sqlite.connect(HOMEDB)
            cur = con.cursor()
            _uni = lambda x: x.decode('utf8', 'ignore')
            cur.execute(''' SELECT folder.path, perm.write
                            FROM folder, perm
                            WHERE folder.id = perm.folder AND
                                owner = ? AND name = ? AND
                                (perm.sharer = ? OR perm.sharer = ?)
                            ORDER BY folder.owner ASC, folder.name ASC
                        ''', (_uni(owner), _uni(name), _uni(login), u'*'))
            result = cur.fetchone()
            con.close()
            if not result:
                raise PathError(404)
            folder, nil, nil = url2path(result[0], owner)
            write = result[1]
            path = os.path.join(folder, child)
    elif path.startswith(HOMES) and login == ADMIN:
        path = path.replace(HOMES, REALHOMES, 1)
    elif path.startswith(DEVICE) and login == ADMIN:
        path = path.replace(DEVICE, REALDEVICE, 1)
        if path.startswith(os.path.join(REALDEVICE, 'iDisk')):
            write = False
    elif path.startswith(DOWNLOADS) and login == ADMIN:
        path = path.replace(DOWNLOADS, REALDOWNLOADS, 1)
    elif path.startswith(PUBLISH):
        # /dav/publish/uuid
        from nas.publish import resolveuuid
        path = path.replace(PUBLISH, '', 1)[1:]
        uuid, sep, child = path.partition(os.sep)
        if not uuid:
            raise PathError(403)
        path, guest, expire, write = resolveuuid(uuid)
        if path:
            path = os.path.normpath(os.path.join(path, child))
        else:
            raise PathError(403)
    else:
        raise PathError(403)

    if kw and kw.get('thumbnail', None) is not None:
        _dir, _file = os.path.split(path)
        path = '%s/.thumbnail/%s.jpg' % (_dir, _file)
    elif kw and kw.get('webview', None) is not None:
        _dir, _file = os.path.split(path)
        path = '%s/.webview/%s.jpg' % (_dir, _file)

    if write is None:
        write = _canwrite(path)

    if not check:
        return (path, None, write)

    try:
        mode = os.stat(path).st_mode
    except os.error as error:
        if error.errno == errno.ENOENT:
            raise PathError(404)
        raise PathError(403)

    if stat.S_ISDIR(mode):
        path += '/'
        return (path, True, write)
    elif stat.S_ISREG(mode):
        return (path, False, write)

    raise PathError(404)

def getchildren(path, login, write, depth):
    '''
    Get the entries in the directory.

    [(relativepath, isdir, writable), ...]

    '''

    children = []
    path = os.path.normpath(path)
    print 'getchildren', 'path', path, 'depth', depth

    if path == os.path.normpath(ROOT):
        children.append(('', None, None))
        if depth == '0':
            return children
        children.append((HOME, None, None))
        children.append((PUBLIC, None, None))
        children.append((SHARE, None, None))
        if login == ADMIN:
            children.append((DEVICE, None, None))
            children.append((DOWNLOADS, None, None))
    elif path == os.path.normpath(SHARE):
        print 'path', path, 'depth', depth
        children.append(('', None, None))
        if depth == '0':
            return children
        con = sqlite.connect(HOMEDB)
        cur = con.cursor()
        cur.execute(''' SELECT f.id, f.path, f.name, f.owner, p.write
                        FROM folder as f, perm as p
                        WHERE f.id = p.folder AND
                            (p.sharer = ? OR p.sharer = ?)
                        ORDER BY f.owner ASC, f.name ASC
                    ''', (login, u'*'))
        for share in cur.fetchall():
            folder, urlpath, name, owner, write = share
            realpath = getrealpath(urlpath, owner, True)
            if not realpath:
                cur.execute('DELETE FROM folder WHERE id = ?', (folder,))
                cur.execute('DELETE FROM perm WHERE id = ?', (folder,))
                continue
            sharename = '%s@%s' % (name.encode('utf8'), owner.encode('utf8'))
            children.append((sharename, write, realpath))
        con.commit()
        con.close()
    elif path == os.path.normpath(PUBLISH):
        children.append(('', None, None))
    else:
        children.append(('', write, path))
        if depth == '0':
            return children
        if os.path.isdir(path):
            for child in os.listdir(path):
                # ignore hidden path
                if child[:1] == '.':
                    continue
                if path == REALDEVICE:
                    try:
                        mp = os.path.join(path, child)
                        os.system('chattr -i %s' % mp)
                        os.rmdir(mp)
                    except os.error:
                        pass
                    else:
                        continue
                #elif path == REALHOMES and child in (HIDEHOMES, login):
                    #continue
                child_path = os.path.join(path, child)
                children.append((child, _canwrite(child_path), child_path))
        if path == REALDEVICE and _canwrite(REALHOMES):  
            children.append((os.path.split(HOMES)[1], True, REALHOMES))
    return children

def getrealpath(path, login, wanteddir=True, system=False):
    if isinstance(path, unicode):
        path = path.encode('utf8')

    url, sep, query = path.partition('?')

    if path[:4] != '/dav' and path[:7] != '/webdav':
        try:
            sb = os.stat(path)
        except os.error:
            return ''
        realpath = path
        isdir = False
        if stat.S_ISDIR(sb.st_mode):
            isdir = True
    else:
        try:
            realpath, isdir, nil = url2path(url, login, query=query, check=True)
        except PathError as error:
            print >> sys.stderr, repr(error)
            return ''

    if wanteddir is True and not isdir:
        realpath = ''
    elif wanteddir is False and isdir:
        realpath = ''
    return realpath

def sigchld(signum, frame):
    try:
        os.waitpid(-os.getpid(), os.WNOHANG | os.WUNTRACED)
    except os.error as error:
        if error.errno == errno.ECHILD:
            pass

def chown(name, uid, gid=0, recursive=True):
    try:
        _chown(name, uid, gid, recursive=recursive)
    except:
        pass

def _chown(name, uid, gid=0, recursive=True):
    if not os.path.isdir(name):
        return

    try:
        os.chown(name, uid, gid)
        os.chmod(name, 0775)
    except os.error:
        pass

    if not recursive:
        return

    for root, dirs, files in os.walk(name):
        for path in dirs + files:
            fullpath = os.path.join(root, path)
            try:
                os.chown(fullpath, uid, gid)
                if os.path.isdir(fullpath):
                    os.chmod(fullpath, 0775)
                else:
                    os.chmod(fullpath, 0664)
            except os.error: pass    # no support ?

def _all(*iterable):
    for element in iterable:
        if element is None:
            return False
    return True

def _any(*iterable):
    for element in iterable:
        if element is not None:
            return True
    return False

TABLES = ['album', 'item', 'share', 'perm', 'folder', 'perm']
ADDRBOOK = ['groups', 'friends', 'members']

INITADDRBOOK = '''
    DROP TABLE IF EXISTS groups;
    DROP TABLE IF EXISTS friends;
    DROP TABLE IF EXISTS members;

    CREATE TABLE groups(
        id INTEGER PRIMARY KEY,
        owner TEXT NOT NULL,
        name TEXT NOT NULL,
        comment TEXT,
        UNIQUE(owner, name)
    );

    CREATE TABLE friends(
        id INTEGER PRIMARY KEY,
        owner TEXT NOT NULL,
        name TEXT NOT NULL,
        email TEXT,
        server TEXT,
        blog TEXT,
        phone TEXT,
        address TEXT,
        comment TEXT,
        UNIQUE(owner, name)
    );

    CREATE TABLE members(
        id INTEGER PRIMARY KEY,
        groupid INTEGER NOT NULL,
        friendid INTEGER NOT NULL,
        UNIQUE(groupid, friendid)
    );
'''

INITDB = '''
    DROP TABLE IF EXISTS album;
    DROP TABLE IF EXISTS item;
    DROP TABLE IF EXISTS share;
    DROP TABLE IF EXISTS folder;
    DROP TABLE IF EXISTS perm;

    CREATE TABLE album(
        id INTEGER PRIMARY KEY,
        owner TEXT NOT NULL,
        name TEXT NOT NULL COLLATE NOCASE,
        cover TEXT DEFAULT '',
        comment TEXT DEFAULT '',
        creation REAL,
        modification REAL,
        UNIQUE(owner, name)
    );

    CREATE TABLE item(
        id INTEGER PRIMARY KEY,
        album INTEGER NOT NULL,
        name TEXT NOT NULL COLLATE NOCASE,
        path TEXT NOT NULL,
        description TEXT,
        UNIQUE(album, path)
    );

    CREATE TABLE share(
        id INTEGER PRIMARY KEY,
        album INTEGER NOT NULL,
        sharer TEXT NOT NULL,
        UNIQUE(album, sharer)
    );

    CREATE TABLE folder(
        id INTEGER PRIMARY KEY,
        owner TEXT NOT NULL,
        name TEXT NOT NULL COLLATE NOCASE,
        path TEXT NOT NULL,
        UNIQUE(owner, name),
        UNIQUE(owner, path)
    );

    CREATE TABLE perm(
        id INTEGER PRIMARY KEY,
        folder INTEGER NOT NULL,
        sharer TEXT NOT NULL,
        write INTEGER DEFAULT 0,
        UNIQUE(folder, sharer)
    );
'''

INITSESSIONDB = '''
    DROP TABLE IF EXISTS session;

    CREATE TABLE session(
        id INTEGER PRIMARY KEY,
        hash TEXT NOT NULL,
        modulus TEXT,
        public TEXT,
        private TEXT,
        user TEXT,
        time REAL,
        UNIQUE(id, hash)
    );
'''

def check_session():
    mask = os.umask(077)
    con = sqlite.connect(SESSIONDB)
    cur = con.cursor()
    cur.execute(''' SELECT name FROM sqlite_master WHERE type = ?
                ''', ('table',))
    result = [x[0] for x in cur.fetchall()]
    if 'session' not in result:
        cur.executescript(INITSESSIONDB)

    for table in result:
        if table == 'session':
            continue
        cur.execute('''DROP TABLE %s''' % table)

    try:
        cur.execute(''' VACUUM session ''')
    except:
        pass
    con.commit()
    con.close()
    os.umask(mask)

def check_table():
    ''' check for sqlite tables
    '''
    makesure_nasdir()
    mask = os.umask(077)
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT name FROM sqlite_master WHERE type = ?
                ''', ('table',))
    result = [x[0] for x in cur.fetchall()]
    for table in TABLES:
        if table not in result:
            cur.executescript(INITDB)
            break
    for table in ADDRBOOK:
        if table not in result:
            cur.executescript(INITADDRBOOK)
            break
    for table in TABLES + ADDRBOOK:
        try:
            cur.execute(''' VACUUM ? ''', (table,))
        except:
            pass
    con.commit()
    con.close()
    os.umask(mask)

NASDIR = '/home/.nas'
SPOOL_DIR = os.path.join(NASDIR, 'spool')
CUPS_DIR = os.path.join(SPOOL_DIR, 'cups')
SAMBA_DIR = os.path.join(SPOOL_DIR, 'samba')
CUPS_TMP_DIR = os.path.join(CUPS_DIR, 'tmp')


def make_user_dir(pw, name):
    realpath = os.path.join(pw.pw_dir, name)
    _mkdir(realpath)
    chown(realpath, pw.pw_uid, pw.pw_gid, recursive=False)


def make_standard_userdirs(pw):
    make_user_dir(pw, 'Documents')
    make_user_dir(pw, 'Music')
    make_user_dir(pw, 'Videos')
    make_user_dir(pw, 'Photos')


def makesure_nasdir():
    _makesure_dir(NASDIR)
    if not _canwrite(NASDIR):
        argv = ['mount', '-t', 'tmpfs', 'none', NASDIR]
        error, status = POPEN(argv)
        status.close()
    _makesure_dir(SPOOL_DIR, attr=0755)
    _makesure_dir(CUPS_DIR, attr=0710, group='lp')
    _makesure_dir(CUPS_TMP_DIR, attr=01770, group='lp')
    _makesure_dir(SAMBA_DIR, attr=01777)

def _makesure_dir(path, attr=0777, owner='root', group='root'):

    try:
        sb = os.lstat(path)
    except os.error:
        _mkdir(path)
        try:
            os.chmod(path, attr)
            _change_owner(path, owner, group)
        except os.error:
            pass
        return True

    if not stat.S_ISDIR(sb.st_mode):
        os.unlink(path)
        return _makesure_dir(path)

def _mkdir(name):
    try:
        os.mkdir(name)
    except os.error as error:
        if error.errno == errno.EEXIST:
            try:
                os.unlink(name)
            except os.error as error:
                if error.errno == errno.EISDIR:
                    return True
            else:
                return _mkdir(name)

    return True

def _change_owner(path, owner, group):
    try:
        uid = pwd.getpwnam(owner).pw_uid
        gid = grp.getgrnam(group).gr_gid
        os.chown(path, uid, gid)
    except:
        raise os.error('error chown %s' % path)


def getvendor():
    try:
        os.readlink(WEBROOT)
    except os.error:
        return ''
    return os.path.basename(os.path.realpath(WEBROOT))


VENDOR_DOMAINS = {
        'akitio' : 'myakitio.com',
        }
DEFAULT_DOMAIN = 'myisharing.com'

def get_server_domain():
    vendor = getvendor()
    return VENDOR_DOMAINS.get(vendor, DEFAULT_DOMAIN)

def echo(orig):
    pos = 0
    result = ''
    orig = orig.replace('\r\n', '\n')
    orig = orig.replace('\r', '\n')
    for x in xrange(len(orig)):
        if orig[x] == '\a':
            pass
        elif orig[x] == '\b':
            if pos < 1:
                pos = 0
            else:
                pos -= 1
        elif orig[x] == '\c':
            return result
        elif orig[x] in ('\f', '\v'):
            current = result[:pos]
            space = ' ' * len(current[current.rfind('\n')+1:])
            result += '\n%s' % space
            pos = len(result)
        else:
            result = result[:pos] + orig[x] + result[pos+1:]
            pos += 1
    return result

class StatusError(Exception):
    def __init__(self, status):
        self.status = status
        self.headers = [('X-Status-Code', str(status))]
        self.body = []

class PathError(StatusError):
    def __init__(self, status=404):
        self.status = status
        self.headers = []
        self.body = []

# pycrypto doesn't support pkcs
def decrypt(session=None, password=None):
    ''' Decrypt Cipher
        param :  session ->
              :  password -> encoded password in hex string
        return:  decoded password
    '''
    if not _all(session, password):
        raise NASError('Bad Request')

    session = str(session).decode('utf8', 'ignore')

    con = sqlite.connect(SESSIONDB)
    cur = con.cursor()
    cur.execute(''' SELECT modulus, private FROM session WHERE hash = ?
                ''', (session,))
    result = cur.fetchone()
    con.close()

    if not result:
        raise NASError('Bad Request')

    (modulus, private) = [x.encode('utf8') for x in result]

    try:
        e = long(password, 16)
    except:
        raise NASError('Invalid Password')

    d = long(private, 16)
    n = long(modulus, 16)

    m = pow(e, d, n)

    s = ''
    while m > 0:
        x = m & 0xff
        s = '%s%s' % (chr(x), s)
        m >>= 8
        n >>= 8

    if x == 2 and 0 < n < 0xff:
        password = s.split('\x00')[1]
        return password

    raise NASError('Invalid Password')

def backup(filename):
    # if not os.path.exists(filename):
    #     file1 = open(filename, 'a+')
    #     file1.write('')
    #     file1.close()

    try:
        info = os.stat(filename)
    except os.error:
        return None

    flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
    mode = stat.S_IMODE(info.st_mode)

    src = os.open(filename, os.O_RDONLY)
    dst = os.open('%s.nasbak2' % filename, flags, mode)

    fcntl.lockf(dst, fcntl.LOCK_EX)

    error = None
    while True:
        data = os.read(src, 0x1000)
        if not data:
            break
        if os.write(dst, data) != len(data):
            error = True
            break
    os.fsync(dst)
    fcntl.lockf(dst, fcntl.LOCK_UN)

    os.close(dst)
    os.close(src)

    if error:
        raise NASError('Backup Error')

    os.rename('%s.nasbak2' % filename, '%s.nasbak' % filename)

def unbackup(filename):
    try:
        os.unlink('%s.nasbak' % filename)
    except os.error:
        pass

def lock(name, block=True):
    pipe = os.open(name, os.O_WRONLY | os.O_CREAT)
    if block:
        fcntl.lockf(pipe, fcntl.LOCK_EX)
        return pipe

    try:
        fcntl.lockf(pipe, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError:
        os.close(pipe)
        return None
    else:
        return pipe

def unlock(pipe):
    fcntl.lockf(pipe, fcntl.LOCK_UN)
    os.close(pipe)

def write(name, content):
    backup(name)
    flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
    pipe = os.open('%s.2' % name, flags, 0644)
    fcntl.lockf(pipe, fcntl.LOCK_EX)
    code = os.write(pipe, content)
    os.fsync(pipe)
    fcntl.lockf(pipe, fcntl.LOCK_UN)
    os.close(pipe)
    if code != len(content):
        raise NASError('write data error', name)
    os.rename('%s.2' % name, name)
    unbackup(name)

def memuse():
    fd = file('/proc/meminfo')
    mem_usage = {}
    line = fd.readline()
    while line:
        line = line.split()
        mem_usage[line[0].strip(':')] = int(line[1])
        line = fd.readline()
    return mem_usage


# vim: set sta sw=4 et:
