#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 21 Jun 2010 01:45:11 PM CST

import os
import sys

REBOOT = False
from system import POPEN
from system import NASError

try:
    import rpm
except ImportError:
    pass

def getinfo(filename):
    ts = rpm.TransactionSet()
    try:
        fd = os.open(filename, os.O_RDONLY)
        header = ts.hdrFromFdno(fd)
    except os.error:
        return None
    except:
        return None

    info = ()
    for tag in ['name', 'version', 'release', 'arch']:
        info += (header[tag],)
    return info

FD = []
def _tsrun(reason, amount, total, filename, verbose):
    if reason == rpm.RPMCALLBACK_INST_OPEN_FILE:
        FD.append(os.open(filename, os.O_RDONLY))
        return FD[-1]
    elif reason == rpm.RPMCALLBACK_INST_START:
        os.close(FD.pop())

#class UpgradeError(Exception):
#    def __init__(self, reason):
#        self.reason = reason

class FirmwareError(NASError):
    def __init__(self, detail=''):
        self.reason = 'Firmware Error'
        self.detail = detail
        self.args = ('Firmware Error', detail)

class DependError(NASError):
    def __init__(self, depends):
        self.reason = 'Depend Error'
        value = ''
        while depends:
            depend = depends.pop(0)
            value += '%s%s%s %s %s%s%s\n' % (
                    depend[0][0], depend[0][1] and '-' or '', depend[0][1] or '',
                    depend[4] == rpm.RPMDEP_SENSE_REQUIRES and 'requires' or 'conflicts',
                    depend[1][0], depend[1][1] and '-' or '', depend[1][1] or '')
        self.detail = value
        self.args = ('Depend Error', value)

class ProblemError(NASError):
    def __init__(self, problems):
        self.reason = 'Upgrade Error'
        value = ''
        while problems:
            value += '%s\n' % problems.pop(0)[0]
        self.detail = value
        self.args = ('Upgrade Error', value)

def upgrade(argv, dryrun):
    filenames = []
    for arg in argv:
        if os.path.exists(arg):
            filenames.append(arg)

    if not filenames:
        raise FirmwareError

    ts = rpm.TransactionSet()

    prob = rpm.RPMPROB_FILTER_OLDPACKAGE
    prob |= rpm.RPMPROB_FILTER_REPLACENEWFILES
    prob |= rpm.RPMPROB_FILTER_REPLACEOLDFILES
    prob |= rpm.RPMPROB_FILTER_REPLACEPKG
    ts.setProbFilter(prob)

    flag = rpm.RPMTRANS_FLAG_NODOCS
    if dryrun:
        flag |= rpm.RPMTRANS_FLAG_TEST
    ts.setFlags(flag)

    reboot = False
    for filename in filenames:
        try:
            fd = os.open(filename, os.O_RDONLY)
            header = ts.hdrFromFdno(fd)
            os.close(fd)
        except os.error:
            continue
        except rpm.error:
            continue

        depends = header.dsOfHeader()

        for install in ts.dbMatch('name', header['name']):
            if depends.EVR() <= install.dsOfHeader().EVR():
                print >> sys.stderr, header['name'], header['version']

        if header['name'] == 'nas':
            reboot = REBOOT
        ts.addInstall(header, filename, 'u')

    depend = ts.check()
    if depend:
        raise DependError(depend)

    ts.setFlags(flag | rpm.RPMTRANS_FLAG_BUILD_PROBS)
    problems = ts.run(_tsrun, '')
    if problems:
        raise ProblemError(problems)

    if not dryrun:
        (error, pipe) = POPEN('service lighttpd stop')
        pipe.close()
        ts.setFlags(flag)
        problems = ts.run(_tsrun, '')
        (error, pipe) = POPEN('service lighttpd start')
        pipe.close()

    if problems:
        raise ProblemError(problems)

    return reboot

MANUAL = '/var/cache/nas/manual'
AUTOMATIC = '/var/cache/nas/automatic'

def update(auto=True, dryrun=False):
    if auto is False:
        directory = MANUAL
    elif auto is True:
        directory = AUTOMATIC
    else:
        directory = os.getcwd()

    if not os.path.isdir(directory):
        return -1

    rpmfiles = []
    for rpmfile in os.listdir(directory):
        if rpmfile[:1] == '.':
            continue
        path = os.path.join(directory, rpmfile)
        if getinfo(path):
            rpmfiles.append(path)
    if not rpmfiles:
        return 9016

    try:
        reboot = upgrade(rpmfiles, dryrun)
    except (FirmwareError, ProblemError):
        for rpmfile in rpmfiles:
            os.unlink(rpmfile)
        if auto:
            return 9018
        else:
            raise
    except DependError:
        for rpmfile in rpmfiles:
            os.unlink(rpmfile)
        if auto:
            return 9010
        else:
            raise

    if dryrun:
        return 0

    for rpmfile in rpmfiles:
        os.unlink(rpmfile)

    (error, pipe) = POPEN('chkconfig nas on')
    pipe.close()
    (error, pipe) = POPEN('chkconfig nasled on')
    pipe.close()

    if reboot and not auto:
        (error, pipe) = POPEN('reboot')
        pipe.close()

    return reboot

# vim: set sta sw=4 et:
