import struct
import cStringIO
import logging

def nas_online(iostream, regkey, seckey, redirected_list=None, last_server_id=None, keep_alive=None, callback=None):
    params = [('key', regkey), ('sec_key', seckey)]
    if redirected_list:
        tmp = ','.join(["%s:%s" % (host, port) for host, port in redirected_list])
        params.append(('redirected_list', tmp))
    if last_server_id:
        params.append(('last_server_id', last_server_id))
    if keep_alive:
        params.append(('keep_alive', keep_alive))
    _send_command(iostream, params, callback)


def send_connect_back_cookie(iostream, cookie, callback=None):
    params = [ ('cookie', cookie) ]
    _send_command(iostream, params, callback)


def _send_command(iostream, params, callback=None):
    data = '\r\n'.join(['%s: %s' % (k.lower(), v) for k, v in params]) + '\r\n\r\n'
    logging.debug("sending command: " + data)
    iostream.write(data, callback)


def read_response(iostream, callback):

    def _callback(params):
        status = params.get('status', '')
        if not status:
            raise ProtocolError('Bad response: no status found')
        callback(int(status.split()[0]), params)

    _do_read(iostream, _callback)


def read_server_command(iostream, callback):
    _do_read(iostream, callback)


def _do_read(iostream, callback):

    def _read_callback(data):
        params = [line.partition(':') for line in data.split('\r\n') if line]
        params = [(k.strip().lower(), v.strip()) for k, _, v in params if k]
        callback(dict(params))

    iostream.read_until('\r\n\r\n', _read_callback)


class ProtocolError(Exception):
    pass

