﻿/*
Program Name: L10N (Localization) By JavaScript
Program URI: http://plugins.jquery.com/project/l10n-by-javascript
Description: This is a very useful script based on JavaScript and XML. You can use it to the localization of the webpage without refresh.

Copyright (c) 2008 Hpyer (coolhpy[at]yahoo.com)
Dual licensed under the MIT (MIT-LICENSE.txt)
and GPL (GPL-LICENSE.txt) licenses.
*/

(function($){
	if(!$.isFunction($.sprintf)||!$.isFunction($.vsprintf)){
		var formats={
			"%":function(val){return"%"}
			,b:function(val){return parseInt(val,10).toString(2)}
			,c:function(val){return String.fromCharCode(parseInt(val,10))}
			,d:function(val){return parseInt(val,10)?parseInt(val,10):0}
			,u:function(val){return Math.abs(val)}
			,f:function(val,p){return(p>-1)?Math.round(parseFloat(val)*Math.pow(10,p))/Math.pow(10,p):parseFloat(val)}
			,o:function(val){return parseInt(val,10).toString(8)}
			,s:function(val){return val}
			,x:function(val){return(""+parseInt(val,10).toString(16)).toLowerCase()}
			,X:function(val){return(""+parseInt(val,10).toString(16)).toUpperCase()}
		};
		var re=/%(?:(\d+)?(?:\.(\d+))?|\(([^)]+)\))([%bcdufosxX])/g;
		var dispatch=function(data){
			if(data.length==1&&typeof data[0]=="object"){
				data=data[0];
				return function(match,w,p,lbl,fmt,off,str){
					return formats[fmt](data[lbl])
				}
			}else{
				var idx=0;
				return function(match,w,p,lbl,fmt,off,str){
					return formats[fmt](data[idx++],p)
				}
			}
		};
		$.extend({
			sprintf:function(format){var argv=Array.apply(null,arguments).slice(1);return format.replace(re,dispatch(argv))}
			,vsprintf:function(format,data){return format.replace(re,dispatch(data))}
		})
	}
	$.extend({
		l10n_picker:function(opts){
			function do_push(words){
				if(words&&$.inArray(words,msgs)==-1){msgs.push(words)}
			}
			function by_selector(){
				var msgid,none,single,plural,tag;
				$("<div>"+opts.html+"</div>").find(opts.selector).each(function(){
					msgid=this.getAttribute("msgid");
					if(!msgid){
						if(tag=="input"&&(-1!=$.inArray(this.type.toLowerCase(),["button","submit"]))){
							msgid=this.value
						}else{
							if(tag=="img"){msgid=this.alt}
							else{msgid=this.innerHTML}
						}
					}
					none=this.getAttribute("none");
					single=this.getAttribute("single");
					plural=this.getAttribute("plural");
					if(none&&single&&plural){
						do_push(none);
						do_push(single);
						do_push(plural)
					}else{
						do_push(msgid)
					}
					if(this.getAttribute("title")){
						msgid=this.getAttribute("msgid_title");
						if(!msgid){
							msgid=this.getAttribute("title")
						}
						none=this.getAttribute("none_title");
						single=this.getAttribute("single_title");
						plural=this.getAttribute("plural_title");
						if(none&&single&&plural){
							do_push(none);
							do_push(single);
							do_push(plural)
						}else{
							do_push(msgid)
						}
					}
				})
			}
			function by_functions(){
				$.each(["__","_e","_n","_ne"],function(i,name){
					var params=1;
					if(i>=2){params=3}
					var reg=new RegExp(this+"\\(\\s*(\"|')+[^)]*\\1(,s*.+)*\\s*\\)","g");
					var funcs=opts.html.match(reg);
					if(funcs&&funcs.length>0){
						$.each(funcs,function(){
							var re=new RegExp("^"+name+"\\(|\\)$","g");
							var rs=this.replace(re,"");
							var arr=eval("(["+rs+"])");
							$.each(arr,function(j,words){
								if(j<params){do_push(words)}
							})
						})
					}
				})
			}
			function to_xml(){
				var str='<?xml version="1.0" encoding="'+opts.encode+'"?>\n\n<messages>\n';
				$.each(msgs,function(){
					str+="\t<message>\n\t\t<msgid>"+this+"</msgid>\n\t\t<msgstr><![CDATA[]]></msgstr>\n\t</message>\n"
				});
				str+="</messages>";
				return str
			}
			opts=$.extend({
				html:$("html").html()
				,selector:""
				,encode:"UTF-8"
				,functions:false
			},opts);
			var msgs=[];
			by_selector();
			if(opts.functions){by_functions()}
			return to_xml()
		}
		,l10n:{
			opts:{
				dir:"./languages"
				,lang:"auto"
				,debug:false
				,before:function(){}
				,after:function(){}
			}
			,messages:[]
			,__:function(words,id){
				if(typeof (id)=="string"){$("#"+id).attr("msgid",words)}
				return this.translate(words)
			}
			,_e:function(words,id){
				document.write(this.__(words,id))
			}
			,_n:function(none,single,plural,number,id){
				if(typeof (id)=="string"){
					$("#"+id).attr("msgid",number).attr("none",none).attr("single",single).attr("plural",plural)
				}
				var words;
				if(!number||number<=0){words=none}
				else{
					if(number==1){words=single}
					else{words=plural}
				}
				return;
			}
			,_ne:function(none,single,plural,number,id){
				document.write(this._n(none,single,plural,number,id))
			}
			,error:function(info){
				if(this.opts.debug&&typeof console!="undefined"){console.log(info)}
			}
			,translate:function(words){
				if(this.opts.lang&&this.messages[this.opts.lang]&&this.messages[this.opts.lang][words]){
					words=this.messages[this.opts.lang][words]
				}
				return words;
			}
			,load:function(L){
				if(L){
					var l=L;
				}else{
					var l=this.opts.lang;
					if(!this.dirs)this.dirs=[]; 
					if(!this.langs)this.langs=[]; 
					var dirsno=$.inArray(this.opts.dir,this.dirs)
					if(dirsno>-1){
						if(this.langs[dirsno]===this.opts.lang){
							$.l10n.callback();
							return;
						}else{
							this.langs[dirsno]=this.opts.lang;
						}
					}else{ 
						this.dirs.push(this.opts.dir);
						this.langs.push(this.opts.lang);
					}
				}
				var uri=this.opts.dir+"/"+l+".xml";	
				$.ajax({
					url:uri
					,dataType:'xml'
					,cache:true
					,async:false
					,success:$.l10n.callback
					,error:function(){
						if(!L&&$.l10n.opts.lang!='en-us'){
							$.l10n.load('en-us');
						}else{
							$.l10n.callback($('<messages></messages>'));
						}
					}
				});
			}

			,callback:function(root){
				//$.l10n.error("Loading completed");
				$.l10n.lang_msg=$.l10n.opts.lang;
				if(!$.l10n.messages[$.l10n.opts.lang])$.l10n.messages[$.l10n.opts.lang]=[];
				var msgs=$("message",root);
				if(msgs.size()==0){
					//$.l10n.error("Invalid language file: "+$.l10n.opts.lang);
					return false
				}
				//$.l10n.error("Parsing...");
				$.each(msgs,function(){
					var msgid,msgstr;
					msgid=$("msgid",this).text();
					msgstr=$("msgstr",this).text();
					/*if(window.ActiveXObject){
						var res=msgid.match(/<(.+)>([^<]*)<\/\1>/g);
						if(res){$.each(res,function(){
							var arr=/<(.+)>([^<]*)<\/\1>/g.exec(this);
							var tag=arr[1].toUpperCase();
							msgid=msgid.replace(new RegExp(arr[0],"g"),"<"+tag+">"+arr[2]+"</"+tag+">")
							})
						}
					}*/
					if(msgid&&msgstr){
						$.l10n.messages[$.l10n.opts.lang][msgid]=msgstr;
					}
				});
				//$.l10n.error("Parsing completed");
			}
			
			,init:function(opts){
				if(typeof opts=="object"){
					$.extend(this.opts,opts);
					this.error("Options have been changed");
				}
				this.opts.lang=this.opts.lang.toLowerCase();
				if(/^ja/.test(this.opts.lang))this.opts.lang='ja';
				if(this.opts.lang=="auto"){
					this.opts.lang=(window.navigator.systemLanguage?window.navigator.systemLanguage:window.navigator.language).toLowerCase();
				}
				if(this.opts.lang!=""){this.load();}
			}
		}
	}
);

$.fn.extend({
	l10n:function(opts){
		$.l10n.opts.before();
		$.l10n.init(opts);
		var msgid,words,none,single,plural,tag;
		this.each(function(){
			msgid=this.getAttribute("msgid");
			msgid_title=this.getAttribute("msgid_title");
			if(msgid===$.l10n.__(msgid)&&msgid_title===$.l10n.__(msgid_title)) return;
			tag=this.tagName.toLowerCase();
			if(!msgid){
				if(tag=="input"&&(-1!=$.inArray(this.type.toLowerCase(),["button","submit"]))){
					msgid=this.value
				}else{
					if(tag=="img"){msgid=this.alt;}
					else{msgid=this.innerHTML;}
				}
			}
			none=this.getAttribute("none");
			single=this.getAttribute("single");
			plural=this.getAttribute("plural");
			if(none&&single&&plural){
				words=$.l10n._n(none,single,plural,msgid)
			}else{
				words=$.l10n.__(msgid);
			}
			this.setAttribute("msgid",msgid);
			if(tag=="input"){
				if($(this).attr("placeholder")){
					$(this).attr("placeholder",$.l10n.__(msgid));
				}else{
					this.value=words;
				}
			}
			else{
				if(tag=="img"){this.alt=words}
				else{this.innerHTML=words}
			}
			if(this.getAttribute("title")){
				msgid=this.getAttribute("msgid_title");
				if(!msgid){msgid=this.getAttribute("title");}
				none=this.getAttribute("none_title");
				single=this.getAttribute("single_title");
				plural=this.getAttribute("plural_title");
				if(none&&single&&plural){words=$.l10n._n(none,single,plural,msgid);}
				else{words=$.l10n.__(msgid)}
				this.setAttribute("msgid_title",msgid);
				this.title=words;
			}
		});
		$.l10n.opts.after();
	}
})})(jQuery);